; Uses a chunk of the PCjr BIOS POST to draw the IBM logo outside of POST.
; Waits for a keystroke, then exits to DOS.
;
; This trickery is achieved by copying the ROM BIOS to RAM so that we can
; patch something to RETF instead of RET.  IBM's routine "borrows" ROM BASIC's
; PRINT routine at INT 82h so we have to set that up as well.
; We copy nearly the entire 64K ROM BIOS/POST/BASIC so that any unknown code
; that needs to run will be available to the relocated routine.
;
; trixter@oldskool.org, 20240317

P8086
MODEL TINY

printofs =      1a2ah
putlogoofs =    0c21h
endlogoofs =    0c49h
_retf =         00CBh

DATASEG
;<<Any initialized data is defined here>>
printf  dd printofs
putlogo dd putlogoofs
endlogo dd endlogoofs

UDATASEG
;<<Any uninitialized data is defined here>>

CODESEG
        STARTUPCODE
        push    cs                      ;save segment of PSP for later
        mov     ax,cs
        add     ax,20h                  ;put our copy 512 bytes past CS/PSP

        ;inject our "malloc'd" 64K area into our pointer vars
        mov     [word ptr printf+2],ax
        mov     [word ptr putlogo+2],ax
        mov     [word ptr endlogo+2],ax

        ;copy ROM BIOS to RAM so we can patch it
        mov     es,ax
        xor     di,di                   ;es:di = destination for BIOS copy
        push    ds
        mov     cx,8000h - 200h         ;copy first 63K (don't need last 1k)
        mov     ax,0f000h
        mov     ds,ax
        xor     si,si                   ;ds:si = F000:0000 (ROM BIOS)
        rep     movsw
        pop     ds

        ;Install print routine at int 82h, needed by BIOS PUT_LOGO
        les     di,printf               ;load our pointer to int82h printcode
        push    ds
        xor     ax,ax
        mov     ds,ax
        mov     [ds:(82h shl 2)],di
        mov     [ds:(82h shl 2)+2],es   ;now INT 82h points to ROM printcode
        pop     ds

        ;Modify RET to RETF in BIOS copy
        les     di,endlogo
        mov     byte ptr es:[di],_retf

        ;set 40-col video mode, then call printlogo routine in our BIOS copy
        mov     ax,01
        int     10h
        call    putlogo

        ;if small message supplied on command-line, print it
        mov     ah,02                   ;move cursor; dh=row, dl=col
        mov     dx,1421h                ;cursor loc where POST prints
        int     10h
        pop     ds
        mov     si,80h                  ;ds:si = command-line pascal string
        mov     cl,[si]                 ;get length
        or      cl,cl                   ;non-zero?
        jz      nomsg                   ;skip if no string to print
        dec     cl                      ;len=len-1 to avoid printing 0D (CR)
        inc     si                      ;skip length byte
        inc     si                      ;skip leading space
        mov     ah,2                    ;use DOS STDOUT to print (smaller code)
printpas:
        lodsb
        mov     dl,al
        int     21h
        loop    printpas

nomsg:
        ;wait for key
        xor     ax,ax
        int     16h
        mov     ax,3
        int     10h

        ;Exit to DOS
        ret

END
